# == Schema Information
#
# Table name: collections
#
#  id                     :integer          not null, primary key
#  name                   :string
#  description            :text
#  privacy                :string           default("private")
#  owner_id               :integer
#  owner_type             :string
#  created_at             :datetime         not null
#  updated_at             :datetime         not null
#  views_count            :integer          default("0")
#  downloads_count        :integer          default("0")
#  likes_count            :integer          default("0")
#  shares_count           :integer          default("0")
#  score                  :float            default("0.0")
#  follows_count          :integer          default("0")
#  thumbnail_file_name    :string
#  thumbnail_content_type :string
#  thumbnail_file_size    :integer
#  thumbnail_updated_at   :datetime
#

class Collection < ApplicationRecord
  include Reviewable
  include Sociable
  include Followable
  include Scoreable
  include Thumbnailable
  include Taggable
  include Trackable
  include Highlights

  has_many :collection_items, as: :collectionable, dependent: :destroy
  has_many :collections, through: :collection_items, source: :collectionable, source_type: 'Collection'
  has_many :learning_objects, through: :collection_items, source: :collectionable, source_type: 'LearningObject'
  has_many :collection_items

  belongs_to :owner, polymorphic: true

  validates_presence_of :name, :owner
  validates_inclusion_of :privacy, in: %w(public private), message: "Privacy must be public or private"

  scope :from_user, ->(user) { where(owner: user) }

  searchkick language: 'brazilian', match: :word_start, searchable: [:name, :description, :author], callbacks: :async

  acts_as_paranoid
  has_paper_trail

  def search_data
    {
        name: name,
        description: description,
        privacy: privacy,
        owner: owner.name,
        score: score,
        created_at: created_at,
    }
  end

  def owner?(candidate)
    owner == candidate
  end

  def user_own?(user)
    return true if user.is_a? Institution and owner.users.include?(user)
    return false unless user.is_a? User
    user.is_admin? || owner?(user)
  end

  def private?
    privacy == 'private'
  end

  ## score methods
  def user_category
    owner.try('user_category')
  end
end
